@@ -370,7 +370,8 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
     self.RemoteCtlSend("%s;1\n" % app.lastBand)	
     # Mode
     btn = app.modeButns.GetSelectedButton()
-    self.RemoteCtlSend("%s;%d\n" % (btn.idName, btn.GetIndex()))
+    if btn:
+      self.RemoteCtlSend("%s;%d\n" % (btn.idName, btn.GetIndex()))
     # Filter and adjustable bandwidth
     name = "Filter 6Slider"
     value = app.midiControls[name][0].button.slider_value
@@ -394,7 +395,7 @@ class ControlCommon(BaseHardware):	# This is the Hardware class for the control
     for idName in ("Mute", "NR2", "AGC", "Sqlch", "NB 1", "Notch", "Test 1", "Spot", "FDX", "PTT", "VOX"):
       self.RemoteCtlSend("%s;%d\n" % (idName, app.idName2Button[idName].GetIndex()))
     # Menus
-    for menu in (app.NB_menu, app.split_menu, app.freedv_menu, app.smeter_menu):
+    for menu in (app.NB_menu, app.split_menu, app.smeter_menu):
       if menu:
         for nid in menu.id2data:
           menu_item = menu.FindItemById(nid)
