@@ -8,7 +8,6 @@
 #include <sys/types.h>
 #include "microphone.h"
 #include "filter.h"
-#include "freedv.h"
 
 #ifdef MS_WINDOWS
 #include <winsock2.h>
@@ -75,6 +74,8 @@ static int vox_level = CLIP16;			// VOX trigger level as a number 0 to CLIP16
 static int timeVOX = 2000;				// VOX hang time in milliseconds
 static int tx_sample_rate = 48000;	// Used for SoapySDR
 static int reverse_tx_sideband;
+static int PsEnable = 0;
+static int PsCal = 0;
 
 static int doTxCorrect = 0;				// Corrections for UDP sample transmit
 static double TxCorrectLevel;
@@ -243,11 +244,9 @@ static void init_alc(struct alc * pt, int size)
 			case DGT_U:
 			case DGT_L:
 			case DGT_IQ:
-				pt->gain_now[i] = 1.4;
-				break;
 			case FDV_U:
 			case FDV_L:
-				pt->gain_now[i] = 2.0;
+				pt->gain_now[i] = 1.4;
 				break;
 			default:
 				pt->gain_now[i] = 1.0;
@@ -372,7 +371,7 @@ static void process_alc(complex double * cSamples, int count, struct alc * pt, r
 
 static int tx_filter(complex double * filtered, int count)
 {	// Input samples are creal(filtered), output is filtered.  The input rate must be 8000 or 48000 sps.
-	int i, is_ssb;
+	int i, is_ssb, mic_interp;
 	int sample_rate = 8000;
 	double dsample, dtmp, magn;
 	complex double csample;
@@ -423,9 +422,10 @@ static int tx_filter(complex double * filtered, int count)
 		}
 		return 0;
 	}
+	mic_interp = MIC_OUT_RATE / sample_rate;
 	// check size of dsamples[] and csamples[] buffer
 	if (count > samples_size) {
-		samples_size = count * 2;
+		samples_size = count * 2 * mic_interp;
 		if (dsamples)
 			free(dsamples);
 		if (csamples)
@@ -541,8 +541,8 @@ static int tx_filter(complex double * filtered, int count)
 		// remove clipping distortion
 		count = quisk_cDecimate(csamples, count, &cfiltAudio3, 1);
 		// Interpolate up to 48000
-		if (MIC_OUT_RATE != sample_rate)
-			count = quisk_cInterpolate(csamples, count, &cfiltInterp, MIC_OUT_RATE / sample_rate);
+		if (mic_interp != 1)
+			count = quisk_cInterpolate(csamples, count, &cfiltInterp, mic_interp);
 		// convert back to 16 bits
 		for (i = 0; i < count; i++) {
 			filtered[i] = csamples[i] * CLIP16;
@@ -573,8 +573,8 @@ static int tx_filter(complex double * filtered, int count)
 		// remove clipping distortion
 		count = quisk_dFilter(dsamples, count, &dfiltAudio3);
 		// Interpolate up to 48000
-		if (MIC_OUT_RATE != sample_rate)
-			count = quisk_dInterpolate(dsamples, count, &dfiltInterp, MIC_OUT_RATE / sample_rate);
+		if (mic_interp != 1)
+			count = quisk_dInterpolate(dsamples, count, &dfiltInterp, mic_interp);
 		// convert back to 16 bits
 		for (i = 0; i < count; i++) {
 			filtered[i] = dsamples[i] * CLIP16;
@@ -614,7 +614,7 @@ static int tx_filter_digital(complex double * filtered, int count)
 		quisk_filt_dInit(&filter1, quiskDgtFilt48Coefs, sizeof(quiskDgtFilt48Coefs)/sizeof(double));    // pass 1350, stop 1650
 	}
 	if ( ! filtered) {	// Change to rxMode
-		quisk_filt_tune(&filter1, 1650.0 / 48000, rxMode != DGT_L && rxMode != LSB);
+		quisk_filt_tune(&filter1, 1650.0 / 48000, rxMode != DGT_L && rxMode != LSB && rxMode != FDV_L);
 		return 0;
 	}
 	for (i = 0; i < count; i++)	// FIR bandpass filter; separate into I and Q
@@ -623,123 +623,6 @@ static int tx_filter_digital(complex double * filtered, int count)
 	return count;
 }
 
-static int tx_filter_freedv(complex double * filtered, int count, int encode)
-{	// Input samples are creal(filtered), output is filtered.
-	// This filter is used for digital voice.
-	// changes by Dave Roberts, G8KBB, for additional modes June, 2020.
-	int i;
-	// The input sample rate is mic_sample_rate, either 8000 or 48000 sps.
-	// The FreeDV codec requires an input n_speech_sample_rate of either 8000 or 16000 sps depending on the mode. Input is always real.
-	// The FreeDV codec will output samples at n_modem_sample_rate of 8000, 16000 or 48000 sps depending on the mode, and output may be real or complex.
-	double dtmp, magn, dsample;
-	static int samples_size = 0;
-	static int last_rx_mode;
-	static int last_freedv_mode;
-	static int do_init = 1;
-	static double aaa, bbb, ccc, Xmin, Xmax, Ymax;
-	static double time_long, time_short;
-	static double inMax=0.3;
-	static double * dsamples = NULL;
-	static struct quisk_cFilter filter2;
-	static struct quisk_dFilter filtDecim;
-	static struct quisk_cFilter cfiltInterp;
-
-	if (do_init) {		// initialization
-		//QuiskWavWriteOpen(&hWav, "jim.wav", 3, 1, 4, 8000, 1.0 / CLIP16);
-		do_init = 0;
-		last_rx_mode = -1;
-		last_freedv_mode = -1;
-		memset(&filter2, 0, sizeof(filter2));
-		quisk_filt_dInit(&filtDecim,  quiskLpFilt48Coefs, sizeof(quiskLpFilt48Coefs)/sizeof(double));   // pass 3000, stop 4000
-		quisk_filt_cInit(&cfiltInterp, quiskLpFilt48Coefs, sizeof(quiskLpFilt48Coefs)/sizeof(double));
-		Ymax = pow(10.0,  - 1 / 20.0);				// maximum y
-		Xmax = pow(10.0,  3 / 20.0);				// x where slope is zero; for x > Xmax, y == Ymax
-		Xmin = Ymax - fabs(Ymax - Xmax);			// x where slope is 1 and y = x; start of compression
-		//printf ("Xmin %f\n", Xmin);
-		aaa = 1.0 / (2.0 * (Xmin - Xmax));			// quadratic
-		bbb = -2.0 * aaa * Xmax;
-		ccc = Ymax - aaa * Xmax * Xmax - bbb * Xmax;
-	}
-	if (last_freedv_mode != freedv_current_mode) {		// correct filter2 depends on freedv mode
-		last_freedv_mode = freedv_current_mode;
-		last_rx_mode = -1;	// re-tune new filter
-		if (filter2.cSamples)
-			free(filter2.cSamples);
-		filter2.cSamples = NULL;
-		if (filter2.cpxCoefs)
-			free(filter2.cpxCoefs);
-		filter2.cpxCoefs = NULL;
-		if (filter2.cBuf)
-			free(filter2.cBuf);
-		filter2.cBuf = NULL;
-		switch(freedv_current_mode) {
-		case FREEDV_MODE_700D:		// filter2 is not used
-			break;
-		// TODO: add modes that return real samples that must be converted to complex
-		default:
-			quisk_filt_cInit(&filter2, quiskFilt53D2Coefs, sizeof(quiskFilt53D2Coefs)/sizeof(double));  // pass 1500, stop 1800
-			break;
-		}
-	}
-	if (last_rx_mode != (int)rxMode) {	// change to sideband
-		last_rx_mode = rxMode;
-		if (rxMode == FDV_U)	 // upper sideband
-			quisk_filt_tune((struct quisk_dFilter *)&filter2, 1600.0 / n_modem_sample_rate, 1);
-		else if (rxMode == FDV_L)   // lower sideband
-			quisk_filt_tune((struct quisk_dFilter *)&filter2, 1600.0 / n_modem_sample_rate, 0);
-	}
-	if ( ! filtered)
-		return 0;
-	// check size of dsamples[] buffer
-	if (count > samples_size) {
-		samples_size = count * 2;
-		if (dsamples)
-			free(dsamples);
-		dsamples = (double *)malloc(samples_size * sizeof(double));
-	}
-	// copy to dsamples[]
-	for (i = 0; i < count; i++)
-		dsamples[i] = creal(filtered[i]) / CLIP16;	// normalize to 1.0000
-	// Decimate to 8000 Hz or to 16000 depending on mode (and hence setting of sample rate)
-	if (quisk_sound_state.mic_sample_rate == 48000)
-		count = quisk_dDecimate(dsamples, count, &filtDecim, quisk_sound_state.mic_sample_rate / n_speech_sample_rate);
-	else if (quisk_sound_state.mic_sample_rate == 8000 && n_speech_sample_rate != 8000)
-		QuiskPrintf("Failure to convert input rate in tx_filter_freedv\n");
-	// Measure average peak input audio level and limit
-	dtmp = 1.0 / n_speech_sample_rate;		// sample time
-	time_long   = 1.0 - exp(- dtmp / 3.000);
-	time_short  = 1.0 - exp(- dtmp / 0.005);
-	for (i = 0; i < count; i++) {
-		dsample = dsamples[i];
-		magn = fabs(dsample);
-		if (magn > inMax)
-			inMax = inMax * (1 - time_short) + time_short * magn;
-		else if(magn > mic_agc_level)
-			inMax = inMax * (1 - time_long) + time_long * magn;
-		else
-			inMax = inMax * (1 - time_long) + time_long * mic_agc_level;
-		dsample = dsample / inMax * Xmin * 0.7;
-		magn = fabs(dsample);
-		if (magn < Xmin)
-			dsamples[i] = dsample;
-		else if (magn > Xmax)
-			dsamples[i] = copysign(Ymax, dsample);
-		else
-			dsamples[i] = copysign(aaa * magn * magn + bbb * magn + ccc, dsample);
-		dsamples[i] = dsamples[i] * CLIP16;
-	}
-	//QuiskWavWriteD(&hWav, dsamples, count);
-	if (encode && pt_quisk_freedv_tx)   // Encode audio into digital modulation
-		count = (* pt_quisk_freedv_tx)(filtered, dsamples, count);
-	//quisk_calc_audio_graph(CLIP16, filtered, NULL, count, 0);
-	if (filter2.cSamples)	// convert float samples to complex with an analytic filter
-		count = quisk_cCDecimate(filtered, count, &filter2, 1);
-	// Interpolate up to 48000
-	if (MIC_OUT_RATE != n_modem_sample_rate ) 
-		count = quisk_cInterpolate(filtered, count, &cfiltInterp, MIC_OUT_RATE / n_modem_sample_rate);
-	return count;
-}
-
 PyObject * quisk_get_tx_filter(PyObject * self, PyObject * args)
 {  // return the TX filter response to display on the graph
 // This is for debugging.  Change quisk.py to call QS.get_tx_filter() instead
@@ -876,6 +759,8 @@ static void quisk_hermes_tx_add(complex double * cSamples, int tx_count, int key
 		if (hermes_read_index >= HERMES_TX_BUF_SHORTS)
 			hermes_read_index -= HERMES_TX_BUF_SHORTS;
 	}
+	// Send the transmit samples to the predistortion logic. Modify the samples in place.
+	PreDistort(NULL, NULL, 0, cSamples, tx_count);
 	hermes_num_samples += tx_count;
 	for (i = 0; i < tx_count; i++) {			// Put transmit mic samples into the buffer
 		hermes_buf[hermes_write_index++] = (short)cimag(cSamples[i]);
@@ -891,13 +776,15 @@ void quisk_hermes_tx_send(int tx_socket, int * tx_records)
 {	// Send one UDP block of mic samples using the Metis-Hermes protocol.  Timing is from blocks received, rate is 48k.
 	// If the key is up we send samples anyway, but the samples are zero.
 	int i, j, offset, sent, ratio;
+	int PsInUse;
 	short s;
 	unsigned char sendbuf[1032];
 	unsigned char * pt_buf;
 	static unsigned int seq = 0;
 	static unsigned char C0_index = 0;
-	unsigned int hlwp = 0;
 	complex double cw_samples[63 * 2];
+	static double writequeue_time0 = 0;
+	static unsigned char rx1_freq[4];	// frequency of the first receiver
 
 	//printf("hermes_tx_send start 1: hermes_num_samples %d\n", hermes_num_samples);
 	if (tx_records == NULL) {
@@ -910,6 +797,7 @@ void quisk_hermes_tx_send(int tx_socket, int * tx_records)
 		last_play_state = quisk_play_state;
 		serial_key_samples(NULL, 0);
 	}
+	PsInUse = PsEnable || PsCal;
 	//printf("hermes_tx_send start 2: hermes_num_samples %d\n", hermes_num_samples);
 	ratio = quisk_sound_state.sample_rate / 48000;		// send rate is 48 ksps
 	//printf ("quisk_hermes_tx_send ratio %d count %d\n", ratio, *tx_records);
@@ -953,15 +841,33 @@ void quisk_hermes_tx_send(int tx_socket, int * tx_records)
 	sendbuf[13] = quisk_pc_to_hermes[offset++];		// C2
 	sendbuf[14] = quisk_pc_to_hermes[offset++];		// C3
 	sendbuf[15] = quisk_pc_to_hermes[offset++];		// C4
-	if (C0_index == 0) {	// Do not change receiver count without stopping Hermes and restarting
+	switch (C0_index) {
+	case 0:	// Do not change receiver count without stopping Hermes and restarting
 		sendbuf[15] = quisk_multirx_count << 3 | 0x04;	// Send the old count, not the changed count
 		if (hermes_mox_bit)		// send filter selection on J16
 			sendbuf[13] = hermes_filter_tx << 1;
 		else
 			sendbuf[13] = hermes_filter_rx << 1;
+		break;
+	case 2:		// Rx1 frequency
+		rx1_freq[0] = sendbuf[12];
+		rx1_freq[1] = sendbuf[13];
+		rx1_freq[2] = sendbuf[14];
+		rx1_freq[3] = sendbuf[15];
+		break;
+	case 3:		// Rx2 frequency
+		if (PsInUse && hermes_mox_bit) {
+			sendbuf[12] = rx1_freq[0];
+			sendbuf[13] = rx1_freq[1];
+			sendbuf[14] = rx1_freq[2];
+			sendbuf[15] = rx1_freq[3];
+		}
+		break;
+	case 10:
+		if (PsInUse && hermes_mox_bit)
+			sendbuf[13] |= 0x40;	// assert pure signal
+		break;
 	}
-        else if ( ! quisk_is_vna && C0_index == 9) {
-        }
 	if (++C0_index > 16)
 		C0_index = 0;
 	pt_buf = sendbuf + 16;
@@ -984,21 +890,24 @@ void quisk_hermes_tx_send(int tx_socket, int * tx_records)
 	sendbuf[522] = 0x7F;
 
 	// Changes for HermesLite v2 thanks to Steve, KF7O
-	// Add an initial delay in sending ACK request so two successive requests are spaced.
-	if ((quisk_hermeslite_writepointer > 0) && (quisk_hermeslite_writeattempts % 8 == 7)) {
-
+	// Add a delay between ACK requests so two successive requests are spaced.
+	if (writequeue_time0 == 0)
+		writequeue_time0 = QuiskTimeSec() + 0.050;	// initial delay
+	if (quisk_hermeslite_writepointer == 1 && QuiskTimeSec() - writequeue_time0 > 0.020) {
+		writequeue_time0 = QuiskTimeSec();
 		// Only send periodic hermeslite writes in second part of frame
-		hlwp = 5*(quisk_hermeslite_writepointer-1);
-		sendbuf[523] = quisk_hermeslite_writequeue[hlwp++] << 1 | hermes_mox_bit;
-		sendbuf[524] = quisk_hermeslite_writequeue[hlwp++];
-		sendbuf[525] = quisk_hermeslite_writequeue[hlwp++];
-		sendbuf[526] = quisk_hermeslite_writequeue[hlwp++];
-		sendbuf[527] = quisk_hermeslite_writequeue[hlwp++];
+		sendbuf[523] = quisk_hermeslite_writequeue[0] << 1 | hermes_mox_bit;
+		sendbuf[524] = quisk_hermeslite_writequeue[1];
+		sendbuf[525] = quisk_hermeslite_writequeue[2];
+		sendbuf[526] = quisk_hermeslite_writequeue[3];
+		sendbuf[527] = quisk_hermeslite_writequeue[4];
 
 		if ((sendbuf[523] & 0x80) == 0) {
 			// No acknowledge requested so fire and forget
-			quisk_hermeslite_writepointer--;
-			quisk_hermeslite_writeattempts = 0;	
+			quisk_hermeslite_writepointer = 0;
+		}
+		else {
+			quisk_hermeslite_writepointer = 2;
 		}
 	} else {
 		offset = C0_index * 4;		// offset into quisk_pc_to_hermes is C0[7:1] * 4
@@ -1007,27 +916,36 @@ void quisk_hermes_tx_send(int tx_socket, int * tx_records)
 		sendbuf[525] = quisk_pc_to_hermes[offset++];		// C2
 		sendbuf[526] = quisk_pc_to_hermes[offset++];		// C3
 		sendbuf[527] = quisk_pc_to_hermes[offset++];		// C4
-		if (C0_index == 0) {
-			sendbuf[527] = quisk_multirx_count << 3 | 0x04;		// Send the old count, not the changed count
+		switch (C0_index) {
+		case 0:	// Do not change receiver count without stopping Hermes and restarting
+			sendbuf[527] = quisk_multirx_count << 3 | 0x04;	// Send the old count, not the changed count
 			if (hermes_mox_bit)		// send filter selection on J16
 				sendbuf[525] = hermes_filter_tx << 1;
 			else
 				sendbuf[525] = hermes_filter_rx << 1;
+			break;
+		case 2:		// Rx1 frequency
+			rx1_freq[0] = sendbuf[524];
+			rx1_freq[1] = sendbuf[525];
+			rx1_freq[2] = sendbuf[526];
+			rx1_freq[3] = sendbuf[527];
+			break;
+		case 3:		// Rx2 frequency
+			if (PsInUse && hermes_mox_bit) {
+				sendbuf[524] = rx1_freq[0];
+				sendbuf[525] = rx1_freq[1];
+				sendbuf[526] = rx1_freq[2];
+				sendbuf[527] = rx1_freq[3];
+			}
+			break;
+		case 10:
+			if (PsInUse && hermes_mox_bit)
+				sendbuf[525] |= 0x40;	// assert pure signal
+			break;
 		}
-                else if ( ! quisk_is_vna && C0_index == 9) {
-                }
 		if (++C0_index > 16)
 			C0_index = 0;
 	}
-	if (quisk_hermeslite_writepointer > 0) quisk_hermeslite_writeattempts++;
-
-	// Abort after 53/8 ~= 5 retries
-	if ((quisk_hermeslite_writepointer > 0) && (quisk_hermeslite_writeattempts > 53)) {
-		QuiskPrintf("ERROR: Maximum Hermes-Lite write attempts without ACK\n");
-		// Cancel entire write sequence
-		quisk_hermeslite_writepointer = 0;
-		quisk_hermeslite_writeattempts = 0;
-	}
 
 	pt_buf = sendbuf + 528;
 	for (i = 0; i < 63; i++) {		// add 63 samples
@@ -1090,12 +1008,47 @@ static void transmit_udp(complex double * cSamples, int count)
 
 static void transmit_mic_carrier(complex double * cSamples, int count, double level)
 {	// send a CW carrier instead of mic samples
+#if 0
+	// transmit a saw tooth
+	static int up = 1;
+	static int ramp = 0;
+	int i;
+	int max = 5000;
+	static int pause = 5000 / 10;
+	int half_max = (int)(0.1 / 0.2339 * max);
+	for (i = 0; i < count; i++) {
+		cSamples[i] = (double)ramp / max * CLIP16;
+		if (ramp == half_max && pause) {
+			pause--;
+		}
+		else if (up) {
+			if (ramp == max) {
+				up = 0;
+				pause = max / 10;
+			}
+			else {
+				ramp += 1;
+			}
+		}
+		else {
+			if (ramp == 0) {
+				up = 1;
+				pause = max / 10;
+			}
+			else {
+				ramp -= 1;
+			}
+		}
+		//printf("Ramp %d\n", ramp);
+	}
+#endif
 #if 1
 	// transmit a carrier equal to the number of samples
 	int i;
 	for (i = 0; i < count; i++)
 		cSamples[i] = level * CLIP16;
-#else
+#endif
+#if 0
 	// replace mic samples with a sin wave
 	int i;
 	double freq;
@@ -1307,6 +1260,8 @@ int quisk_process_microphone(int mic_sample_rate, complex double * cSamples, int
 		case DGT_L:
 		case DGT_IQ:
 		case DGT_FM:
+		case FDV_U:		// FreeDV
+		case FDV_L:
 			count = tx_filter_digital(cSamples, count);	// filter samples, minimal processing
 			process_alc(cSamples, count, &tx_alc, rxMode);
 			break;
@@ -1314,11 +1269,6 @@ int quisk_process_microphone(int mic_sample_rate, complex double * cSamples, int
 			count *= interp;
 			transmit_mic_imd(cSamples, count, quiskImdLevel / 1000.0);
 			break;
-		case FDV_U:	// FDV
-		case FDV_L:
-			count = tx_filter_freedv(cSamples, count, 1);
-			process_alc(cSamples, count, &tx_alc, rxMode);
-			break;
 		default:
 			break;
 		}
@@ -1409,11 +1359,6 @@ int quisk_process_microphone(int mic_sample_rate, complex double * cSamples, int
 			count = tx_filter(cSamples, count);
 			process_alc(cSamples, count, &tx_alc, rxMode);
 			break;
-		case FDV_U:	// FDV
-		case FDV_L:
-			count = tx_filter_freedv(cSamples, count, 0);
-			process_alc(cSamples, count, &tx_alc, rxMode);
-			break;
 		default:
 			for (i = 0; i < count; i++)
 				cSamples[i] = 0;
@@ -1435,14 +1380,31 @@ int quisk_process_microphone(int mic_sample_rate, complex double * cSamples, int
 	return count;
 }
 
+PyObject * quisk_get_tx_audio(PyObject * self, PyObject * args)
+{
+	const char * name;
+
+	if (!PyArg_ParseTuple (args, "s", &name))
+		return NULL;
+
+	if (strcmp(name, "PsEnable") == 0)
+		return PyInt_FromLong(PsEnable);
+	if (strcmp(name, "PsCal") == 0)
+		return PyInt_FromLong(PsCal);
+	Py_INCREF (Py_None);
+	return Py_None;
+}
+
+
 PyObject * quisk_set_tx_audio(PyObject * self, PyObject * args, PyObject * keywds)
 {  /* Call with keyword arguments ONLY; change Tx audio parameters */
 	static char * kwlist[] = {"vox_level", "vox_time", "mic_clip", "mic_preemphasis", "tx_sample_rate",
-		"reverse_tx_sideband", NULL} ;
+		"reverse_tx_sideband", "PsEnable", "PsCal", NULL} ;
 	int vlevel = -9999, clevel = -9999;
 
-	if (!PyArg_ParseTupleAndKeywords (args, keywds, "|iiidii", kwlist,
-			&vlevel, &timeVOX, &clevel, &quisk_mic_preemphasis, &tx_sample_rate, &reverse_tx_sideband))
+	if (!PyArg_ParseTupleAndKeywords (args, keywds, "|iiidiiii", kwlist,
+			&vlevel, &timeVOX, &clevel, &quisk_mic_preemphasis, &tx_sample_rate, &reverse_tx_sideband,
+			&PsEnable, &PsCal))
 		return NULL;
 	if (vlevel != -9999)
 		vox_level = (int)(pow(10.0, vlevel / 20.0) * CLIP16);	// Convert dB to 16-bit sample
@@ -1566,7 +1528,6 @@ void quisk_set_tx_mode(void)	// called when the mode rxMode is changed
 	tx_filter(NULL, 0);
 	tx_filter_digital(NULL, 0);
 	transmit_udp(NULL, 0);
-	tx_filter_freedv(NULL, 0, 0);
 	serial_key_samples(NULL, 0);
 }
 
@@ -1612,3 +1573,527 @@ static void serial_key_samples(complex double * cSamples, int count)	// called f
 		cSamples[i] = ampl * CLIP16;
 	}
 }
+
+static void spline(double x[], double y[], int n, double yp1, double ypn, double y2[])
+{
+	int i,k;
+	double p,qn,sig,un,*u;
+
+	u = (double *)malloc(n * sizeof(double));
+	if (yp1 > 0.99e30)
+		y2[1]=u[1]=0.0;
+	else {
+		y2[1] = -0.5;
+		u[1]=(3.0/(x[2]-x[1]))*((y[2]-y[1])/(x[2]-x[1])-yp1);
+	}
+	for (i=2;i<=n-1;i++) {
+		sig=(x[i]-x[i-1])/(x[i+1]-x[i-1]);
+		p=sig*y2[i-1]+2.0;
+		y2[i]=(sig-1.0)/p;
+		u[i]=(y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
+		u[i]=(6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1])/p;
+	}
+	if (ypn > 0.99e30)
+		qn=un=0.0;
+	else {
+		qn=0.5;
+		un=(3.0/(x[n]-x[n-1]))*(ypn-(y[n]-y[n-1])/(x[n]-x[n-1]));
+	}
+	y2[n]=(un-qn*u[n-1])/(qn*y2[n-1]+1.0);
+	for (k=n-1;k>=1;k--)
+		y2[k]=y2[k]*y2[k+1]+u[k];
+	free(u);
+}
+
+static void polint(double xa[], double ya[], double x, double *y)
+{
+	int i,m,ns=1;
+	double den,dif,dift,ho,hp,w, dy;
+	double c[4], d[4];
+
+	dif=fabs(x-xa[1]);
+	for (i=1;i<=3;i++) {
+		if ( (dift=fabs(x-xa[i])) < dif) {
+			ns=i;
+			dif=dift;
+		}
+		c[i]=ya[i];
+		d[i]=ya[i];
+	}
+	*y=ya[ns--];
+	for (m=1;m<3;m++) {
+		for (i=1;i<=3-m;i++) {
+			ho=xa[i]-x;
+			hp=xa[i+m]-x;
+			w=c[i+1]-d[i];
+			den = ho - hp;
+			if ( (den=ho-hp) == 0.0)
+				printf("Error in routine polint\n");
+			den=w/den;
+			d[i]=hp*den;
+			c[i]=ho*den;
+		}
+		*y += (dy=(2*ns < (3-m) ? c[ns+1] : d[ns--]));
+	}
+}
+
+static void splint(double xa[], double ya[], double y2a[], int n, double x, double *y)
+{
+	static int klo = 1;
+	int khi,k;
+	double h,b,a;
+
+	khi = klo + 1;
+	if (x < xa[klo] || x > xa[khi]) {
+		klo=1;
+		khi=n;
+		while (khi-klo > 1) {
+			k=(khi+klo) >> 1;
+			if (xa[k] > x) khi=k;
+			else klo=k;
+		}
+	}
+	h=xa[khi]-xa[klo];
+	if (h == 0.0)
+		printf("Bad xa input to routine splint\n");
+	a=(xa[khi]-x)/h;
+	b=(x-xa[klo])/h;
+	*y=a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0;
+}
+
+#define NUM_BINS	21
+#define BIN_SPACING	(1.0 / (NUM_BINS - 1))
+#define BIN_TOL		(BIN_SPACING * 0.20)
+#define BLOCK_MSECS	2000		// Time of one data block.
+#define MILLISECS(x)	(48 * (x))	// Convert milliseconds to samples at 48000 samples per second.
+#define DEBUG_PS	0
+#define HOLD		0		// Make calibration run permanent
+#define RX1_GAIN	0.238
+void PreDistort(complex double * amp_in_samples, complex double * amp_out_samples, int nSamples, complex double * tx_samples, int num_tx)
+{
+	int i, bin, num_bin_data;
+	double G0, phase, mag_in, mag_out, new_mag, aaa, ppp;
+	double error, xp, xpg, xp0, xpg0;
+	double quadX[4], quadY[4], quadZ[4];
+	complex double in_sample, out_sample;
+	static int state = 0;
+	static int Fault;
+	static int isCalibrated = 0;
+	static int ramp_counter, ramp_count0;
+	static int input_samples = 0;
+	static double tx_to_RX1_gain = 0;
+	static double max_out_mag, starting_phase;
+#if DEBUG_PS
+	static double clip_tx, clip_mag_in, clip_aaa;
+#endif
+#if DEBUG_PS > 2
+	static int old_state;
+	static double time0;
+#endif
+	static int spline_P_size;
+	static double spline_P_X[NUM_BINS + 1];
+	static double spline_P_mag[NUM_BINS + 1];
+	static double spline_P_mag2[NUM_BINS + 1];
+	static double spline_P_phase[NUM_BINS + 1];
+	static double spline_P_phase2[NUM_BINS + 1];
+
+	struct BPD {
+		int XP_count;
+		double table_XP;
+		double XP_mag;
+		double mag_X;
+		double mag_P;
+		double phase_P;
+		complex double XP_gain;
+		double XP_gain_mag;
+		double XP_gain_phase;
+	};
+	static struct BPD Bins[NUM_BINS];
+		
+	static enum {
+		NothingHappens,
+		SendCW,
+		StopCW
+	} tx_state = NothingHappens;
+
+	if (quisk_multirx_count == 0) {
+		state = 0;
+		return;
+	}
+	if (PsCal) {
+		if (state == 0) {
+			isCalibrated = 0;
+			state = 10;
+#if DEBUG_PS > 2
+			time0 = QuiskTimeSec();
+#endif
+		}
+	}
+	else if ( ! PsEnable) {
+		state = 0;
+		return;
+	}
+	else if ( ! hermes_mox_bit) {
+		state = 0;
+		return;
+	}
+
+	if (num_tx) {
+		input_samples += num_tx;
+		if (isCalibrated) {	// perform predistortion of Tx samples; multiply input X by P
+			for (i = 0; i < num_tx; i++) {
+				mag_in = cabs(tx_samples[i]) / CLIP16;
+				splint(spline_P_X, spline_P_mag, spline_P_mag2, spline_P_size, mag_in, &aaa);
+				splint(spline_P_X, spline_P_phase, spline_P_phase2, spline_P_size, mag_in, &ppp);
+				tx_samples[i] *= aaa * cexp(I * ppp);
+				new_mag = mag_in * aaa;
+				if (new_mag > 1.0) {
+					tx_samples[i] /= new_mag;
+#if DEBUG_PS
+					if (new_mag > 1.0001 && new_mag > clip_tx) {
+						clip_tx = new_mag;
+						clip_mag_in = mag_in;
+						clip_aaa = aaa;
+					}
+#endif
+				}
+			}
+		}
+		switch (tx_state) {
+		case NothingHappens:
+		default:
+			break;
+		case SendCW:	// Start sending a CW carrier.
+			for (i = 0; i < num_tx; i++) {
+				if (ramp_counter > 0)	// slow increase
+					tx_samples[i] = CLIP16 * (1.0 - (double)ramp_counter-- / ramp_count0);
+				else
+					tx_samples[i] = CLIP16;
+			}
+			break;
+		case StopCW:	// Stop sending a CW carrier.
+			for (i = 0; i < num_tx; i++) {
+				if (ramp_counter > 0)	// slow decrease
+					tx_samples[i] = (double)CLIP16 * ramp_counter-- / ramp_count0;
+				else
+					tx_samples[i] = 0;
+			}
+			break;
+		}
+	}
+
+	if (nSamples <= 0)
+		return;
+
+#if DEBUG_PS > 2
+	if (state != old_state) {
+		aaa = cabs(amp_in_samples[0] / CLIP32);
+		printf("State %2d time %6.3lf sample %12.6f\n", old_state, QuiskTimeSec() - time0, aaa);
+		old_state = state;
+		time0 = QuiskTimeSec();
+	}
+#endif
+
+	switch (state) {
+	case 0:		// Start a timer
+		tx_state = NothingHappens;
+		input_samples = 0;
+		starting_phase = 0;
+		if ( ! HOLD)
+			state++;
+		break;
+	case 1:
+		if (input_samples >= MILLISECS(200)) {
+#if DEBUG_PS
+			clip_tx = 0.0;
+#endif
+			for (bin = 0; bin < NUM_BINS; bin++) {		// initialize
+				Bins[bin].XP_count = 0;
+				Bins[bin].XP_mag = 0;
+				Bins[bin].XP_gain = 0;
+			}
+			input_samples = 0;
+			state++;
+		}
+		break;
+	case 2:
+	case 15:
+		if (input_samples >= MILLISECS(BLOCK_MSECS)) {
+			num_bin_data = 0;
+			Fault = 0;
+			for (bin = 0; bin < NUM_BINS; bin++) {
+				if (Bins[bin].XP_count) {
+					num_bin_data++;
+					Bins[bin].XP_mag /= Bins[bin].XP_count;
+					Bins[bin].XP_gain /= Bins[bin].XP_count;
+					if (starting_phase == 0 && bin >= NUM_BINS / 2) {
+						starting_phase = carg(Bins[bin].XP_gain);
+#if DEBUG_PS
+						printf("starting_phase %.3f\n", starting_phase);
+#endif
+					}
+				}
+			}
+			if (PsCal && num_bin_data < NUM_BINS - 4) {
+				printf("PS calibration failed to fill all bins ");
+				for (bin = 0; bin < NUM_BINS; bin++)
+					if (Bins[bin].XP_count)
+						printf("1");
+					else
+						printf("0");
+				printf("\n");
+				Fault = 1;
+			}
+			state++;
+		}
+		else {
+			for (i = 0; i < nSamples; i++) {
+				in_sample = amp_in_samples[i] / CLIP32 / RX1_GAIN;
+				mag_in = cabs(in_sample);
+				if (tx_to_RX1_gain < mag_in) {
+					tx_to_RX1_gain = mag_in;
+#if DEBUG_PS > 2
+					printf("tx RX1 gain %.6f, max_out_mag %.6f\n", tx_to_RX1_gain, max_out_mag);
+#endif
+				}
+				bin = (int)(mag_in / BIN_SPACING + 0.5);
+				if ( ! bin)
+					continue;
+				error = fabs(mag_in - Bins[bin].table_XP);
+				if (error < BIN_TOL) {
+					out_sample = amp_out_samples[i] / CLIP32;
+					Bins[bin].XP_gain += out_sample / in_sample;
+					Bins[bin].XP_mag += mag_in;
+					Bins[bin].XP_count++;
+				}
+			}
+			if (state == 15) {
+				if (tx_state != StopCW) {
+					ramp_counter = ramp_count0 = MILLISECS(BLOCK_MSECS - 500);
+					tx_state = StopCW;
+				}
+			}
+		}
+		break;
+	case 3:
+	case 16:
+#if DEBUG_PS
+		if (clip_tx != 0.0) {
+			printf ("clip output sample %.6f, mag_in %.6f, predistort %.6f\n", clip_tx, clip_mag_in, clip_aaa);
+			clip_tx = 0.0;
+		}
+#endif
+		for (bin = 1; bin < NUM_BINS - 1; bin++) {
+			if (Bins[bin].XP_count > 0) {	// This bin has data.
+				Bins[bin].XP_gain_mag = cabs(Bins[bin].XP_gain);
+				phase = carg(Bins[bin].XP_gain) - starting_phase;
+				if (phase < - M_PI)
+					phase += 2 * M_PI;
+				else if (phase > M_PI)
+					phase -= 2 * M_PI;
+				Bins[bin].XP_gain_phase = phase;
+			}
+			else {
+				Bins[bin].XP_gain_mag = 1.0;
+				Bins[bin].XP_gain_phase = 0.0;
+			}
+		}
+		Bins[0].XP_count = 1;
+		Bins[0].XP_mag = 0.0;
+		i = 1;
+		for (bin = 1; bin < 6; bin++) {
+			if (Bins[bin].XP_count) {
+				quadX[i] = Bins[bin].XP_mag;
+				quadY[i] = Bins[bin].XP_gain_mag;
+				quadZ[i] = Bins[bin].XP_gain_phase;
+				if (i == 3)
+					break;
+				i++;
+			}
+		}
+		if (i == 3) {
+			polint(quadX, quadY, 0.0, &Bins[0].XP_gain_mag);
+			polint(quadX, quadZ, 0.0, &Bins[0].XP_gain_phase);
+		}
+		else {
+			Bins[0].XP_gain_mag = 1.0;
+			Bins[0].XP_gain_phase = 0.0;
+			Fault = 1;
+			if (DEBUG_PS || PsCal)
+				printf("Failure to find G0 at zero\n");
+		}
+
+		Bins[NUM_BINS - 1].XP_count = 1;
+		Bins[NUM_BINS - 1].XP_mag = 1.0;
+		i = 3;
+		for (bin = NUM_BINS - 2; bin >= NUM_BINS - 6; bin--) {
+			if (Bins[bin].XP_count) {
+				quadX[i] = Bins[bin].XP_mag;
+				quadY[i] = Bins[bin].XP_gain_mag;
+				quadZ[i] = Bins[bin].XP_gain_phase;
+				if (--i == 0)
+					break;
+			}
+		}
+		if (i == 0) {
+			polint(quadX, quadY, 1.0, &Bins[NUM_BINS - 1].XP_gain_mag);
+			polint(quadX, quadZ, 1.0, &Bins[NUM_BINS - 1].XP_gain_phase);
+		}
+		else {
+			Bins[NUM_BINS - 1].XP_gain_mag = 1.0;
+			Bins[NUM_BINS - 1].XP_gain_phase = 0.0;
+			Fault = 1;
+			if (DEBUG_PS || PsCal)
+				printf("Failure to find G0 at one\n");
+		}
+		state++;
+		break;
+	case 4:
+	case 17:
+		G0 = Bins[NUM_BINS - 1].XP_gain_mag;
+		num_bin_data = 0;
+		xp0 = -1.0;
+		xpg0 = -1.0;
+		for (bin = 0; bin < NUM_BINS; bin++) {
+			if (bin == NUM_BINS - 1) {
+				num_bin_data++;
+				Bins[NUM_BINS - 1].mag_P = 1.0;
+				Bins[NUM_BINS - 1].phase_P = - Bins[NUM_BINS - 1].XP_gain_phase;
+				Bins[NUM_BINS - 1].mag_X = 1.0;
+			}
+			else if (Bins[bin].XP_count) {
+				num_bin_data++;
+				Bins[bin].mag_P = G0 / Bins[bin].XP_gain_mag;
+				Bins[bin].phase_P = - Bins[bin].XP_gain_phase;
+				Bins[bin].mag_X = Bins[bin].XP_mag / Bins[bin].mag_P;
+				xp = Bins[bin].mag_X * Bins[bin].mag_P;
+				if (xp < xp0) {
+					if (DEBUG_PS || PsCal)
+						printf("xp is not monotonic at bin %d\n", bin);
+					Fault = 1;
+				}
+				xpg = xp * Bins[bin].XP_gain_mag;
+				if (xpg < xpg0) {
+					if (DEBUG_PS || PsCal)
+						printf("xpg is not monotonic at bin %d\n", bin);
+					Fault = 1;
+				}
+				xp0 = xp;
+				xpg0 = xpg;
+			}
+			else {
+				Bins[bin].mag_P = 0.0;
+				Bins[bin].phase_P = 0.0;
+				Bins[bin].mag_X = 0.0;
+			}
+		}
+		if (num_bin_data < NUM_BINS  * 7 / 10) {
+			if (DEBUG_PS || state == 17)
+				printf("Not enough data for corrections %d\n", num_bin_data);
+			Fault = 1;
+		}
+		if ( ! Fault) {
+#if DEBUG_PS
+			printf("Create a new predistortion\n");
+#endif
+			spline_P_size = 0;
+			for (bin = 0; bin < NUM_BINS; bin++) {
+				if (Bins[bin].XP_count) {
+					spline_P_size++;
+					spline_P_X[spline_P_size] = Bins[bin].mag_X;
+					spline_P_mag[spline_P_size] = Bins[bin].mag_P;
+					spline_P_phase[spline_P_size] = Bins[bin].phase_P;
+				}
+			}
+
+			spline(spline_P_X, spline_P_mag, spline_P_size, 2E30, 2E30, spline_P_mag2);
+			spline(spline_P_X, spline_P_phase, spline_P_size, 2E30, 2E30, spline_P_phase2);
+		}
+		state++;
+		break;
+	case 5:	
+	case 18:
+#if DEBUG_PS
+		if (Fault)
+			printf("Data for predistortion was rejected\n");
+		printf("\n");
+#endif
+		input_samples = 0;
+		state++;
+		break;
+	case 6:
+		if ( ! HOLD)
+			state = 1;
+		input_samples = 0;
+		break;
+	case 10:
+		isCalibrated = 0;
+		input_samples = 0;
+		quisk_set_key_down(1);
+		max_out_mag = 0;
+		tx_to_RX1_gain = 0;
+		starting_phase = 0;
+		ramp_counter = ramp_count0 = MILLISECS(10);
+#if DEBUG_PS
+		clip_tx = 0.0;
+#endif
+		for (bin = 0; bin < NUM_BINS; bin++) {
+			Bins[bin].table_XP = BIN_SPACING * bin;
+			Bins[bin].XP_mag = 0;
+			Bins[bin].XP_count = 0;
+			Bins[bin].XP_gain = 0;
+		}
+		tx_state = SendCW;
+		state++;
+		break;
+	case 11:
+		if (input_samples >= MILLISECS(400)) {
+			input_samples = 0;
+			state++;
+		}
+		if (input_samples >= MILLISECS(50) && cabs(amp_in_samples[0]) / CLIP32 / RX1_GAIN > 0.1) {
+			input_samples = 0;
+			state++;
+		}
+		break;
+	case 12:
+		if (input_samples >= MILLISECS(200)) {
+			state++;
+		}
+		else {
+			for (i = 0; i < nSamples; i++) {
+				in_sample = amp_in_samples[i] / CLIP32;
+				mag_in = cabs(in_sample);
+				if (tx_to_RX1_gain < mag_in)
+					tx_to_RX1_gain = mag_in;
+				out_sample = amp_out_samples[i] / CLIP32;
+				mag_out = cabs(out_sample);
+				if (max_out_mag < mag_out)
+					max_out_mag = mag_out;
+			}
+		}
+		break;
+	case 13:
+#if DEBUG_PS
+		printf("tx RX1 gain %.5f, max_out_mag %.6f\n", tx_to_RX1_gain, max_out_mag);
+#endif
+		if (fabs(tx_to_RX1_gain - RX1_GAIN) / RX1_GAIN > 1E-2)
+			printf("Measured %.6f and assumed %.6f RX1 gain differ\n", tx_to_RX1_gain, RX1_GAIN);
+		input_samples = 0;
+		state++;
+		break;
+	case 14:
+		input_samples = 0;
+		state++;
+		break;
+	case 19:
+		quisk_set_key_down(0);
+		tx_state = NothingHappens;
+		PsCal = 0;
+		if ( ! Fault)
+			isCalibrated = 1;
+		input_samples = 0;
+		state = 0;
+		break;
+	}
+}
