@@ -1,18 +1,64 @@
 # This file supports various hardware boxes at my shack
 
-from __future__ import print_function
-
-import sys, struct, math, socket, select, thread, time, traceback, os
-import cPickle
-import serial
-import _quisk as QS
-import wx
+import sys, struct, math, socket, select, time, traceback, os
+import wx, wx.lib.buttons
 
 DEBUG = 0
 
 gatewayTime = 0			# time of last gateway command
 gatewayLimit = 0.2	# minimum time between gateway commands
 
+class ControlBox:	# Control my station control box
+  address = ('192.168.1.194', 0x3A00 + 64)
+  def __init__(self, app, conf):
+    self.application = app	# Application instance (to provide attributes)
+    self.conf = conf		# Config file module
+    self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+    self.socket.setblocking(0)
+    self.socket.connect(self.address)
+    self.have_data = b'C\000'
+    self.want_data = b'C\000'
+    self.timer = 0
+  def close(self):
+    self.want_data = b'C\000'	# raise key if down
+    if self.have_data != self.want_data:
+      self.socket.send(self.want_data)
+      time.sleep(0.1)
+      self.socket.send(self.want_data)
+  def OnButtonPTT(self, event):
+    btn = event.GetEventObject()
+    if btn.GetValue():		# Turn the software key bit on or off
+      self.want_data = b'C\001'
+    else:
+      self.want_data = b'C\000'
+  def SetKeyDown(self, down):
+    if down:			# Turn the software key bit on or off
+      self.want_data = b'C\001'
+    else:
+      self.want_data = b'C\000'
+  def HeartBeat(self):
+    global gatewayTime
+    if not self.socket:
+      return
+    try:	# The control box echoes its commands
+      self.have_data = self.socket.recv(50)
+    except socket.error:
+      pass
+    except socket.timeout:
+      pass
+    if self.have_data != self.want_data and time.time() - gatewayTime > gatewayLimit:
+      gatewayTime = time.time()
+      if self.timer <= 10:
+        self.timer += 1
+        if self.timer == 10:
+          print ('Control box error')
+      try:
+        self.socket.send(self.want_data)
+      except socket.error:
+        pass
+      except socket.timeout:
+        pass
+
 class LowPassFilter:	# Control my low pass filter box
   address = ('192.168.1.194', 0x3A00 + 39)
   # Filters are numbered 1 thru 8 for bands: 80, 15, 60, 40, 30, 20, 17, short
@@ -28,7 +74,7 @@ class LowPassFilter:	# Control my low pass filter box
     self.socket.setblocking(0)
     self.socket.connect(self.address)
     self.have_data = None
-    self.want_data = '\00'
+    self.want_data = b'\000'
     self.old_tx_freq = 0
     self.timer = 0
   def ChangeBand(self, band):
@@ -44,8 +90,9 @@ class LowPassFilter:	# Control my low pass filter box
       num = self.lpfnum[tx_freq // 1000000]
     except IndexError:
       num = 8
-    self.want_data =  chr(num)
+    self.want_data =  bytearray((num, ))
     self.timer = 0
+    #print ("LP filter band %d" % num)
   def HeartBeat(self):
     global gatewayTime
     if not self.socket:
@@ -79,7 +126,7 @@ class HighPassFilter:	# Control my high pass filter box
     self.socket.setblocking(0)
     self.socket.connect(self.address)
     self.have_data = None
-    self.want_data = '\00\00\00'
+    self.want_data = bytes(3)
     self.old_tx_freq = 0
     self.timer = 0
   def ChangeBand(self, band):
@@ -168,7 +215,7 @@ class HighPassFilter:	# Control my high pass filter box
     else:
       portb = 0x40
     portd |= self.preamp
-    self.want_data =  chr(portb) + chr(portc) + chr(portd)
+    self.want_data = bytearray((portb, portc, portd))
     self.timer = 0
   def HeartBeat(self):
     global gatewayTime
@@ -193,6 +240,115 @@ class HighPassFilter:	# Control my high pass filter box
       except socket.timeout:
         pass
 
+class FilterBoxV2:	# Control my 2016 high/low pass filter box
+  address = ('192.168.1.194', 0x3A00 + 70)
+  # Low pass filters are numbered 0 thru 5 for bands: 10, 15, 17, 20, 40, 60
+  lpfnum = (5, 5, 5, 5, 5, 5,	# frequency 0 thru 5 MHz
+               4, 4, 3, 3, 3,	# 6 thru 10
+               3, 3, 3, 3, 2,	# 11 thru 15
+               2, 2, 2, 1, 1,	# 16 thru 20
+               1, 0, 0, 0, 0)	# 21 thru 25; otherwise the filter is 0
+  # High pass filters are numbered 0 thru 2 with cutoff 12.5 MHz, 4.2 MHz, short
+  hpfnum = (2, 2, 2, 2, 2, 1,	# frequency 0 thru 5 MHz
+               1, 1, 1, 1, 1,	# 6 thru 10
+               1, 1, 0, 0, 0,	# 11 thru 15
+               0, 0, 0, 0, 0,	# 16 thru 20
+               0, 0, 0, 0, 0)	# 21 thru 25; otherwise the filter is 0
+  def __init__(self, app, conf):
+    self.application = app	# Application instance (to provide attributes)
+    self.conf = conf		# Config file module
+    self.preamp = 0
+    self.have_data = None
+    self.want_data = b'\x00\x00'
+    self.old_tx_freq = 0
+    self.timer = 0
+    self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+    self.socket.setblocking(0)
+    self.socket.connect(self.address)
+  def ChangeBand(self, band):
+    pass
+  def DefaultPreamp(self, freq):
+    preamp = 0
+    if self.conf.use_rx_udp:
+      btn = self.application.BtnRfGain
+      if btn:
+        if freq < 5000000:
+          btn.SetLabel('RF 0 dB', False)
+        elif freq < 13000000:
+          btn.SetLabel('RF 0 dB', False)
+        else:
+          btn.SetLabel('RF +16', False)
+          preamp = 1
+    return preamp
+  def OnButtonRfGain(self, event):
+    btn = event.GetEventObject()
+    n = btn.index
+    if n == 0:		# 0dB
+      self.preamp = 0
+      self.application.Hardware.rf_gain = 0
+    elif n == 1:	# +16
+      self.preamp = 1
+      self.application.Hardware.rf_gain = 16
+    else:
+      print ('Unknown RfGain')
+      self.preamp = 0
+      self.application.Hardware.rf_gain = 0
+    if DEBUG: print ('Gain', self.preamp, self.application.Hardware.rf_gain)
+    self.SetTxFreq(None)
+  def SetPreamp(self, preamp):
+    if preamp:
+      self.preamp = 1
+    else:
+      self.preamp = 0
+    if DEBUG: print ("Preamp", preamp)
+    self.SetTxFreq(None)
+  def SetTxFreq(self, tx_freq):
+    if tx_freq is None:
+      tx_freq = self.old_tx_freq
+    elif abs(self.old_tx_freq - tx_freq) < 100000:
+      return	# Ignore small tuning changes
+    if abs(self.old_tx_freq - tx_freq) > 1000000:
+      self.preamp = self.DefaultPreamp(tx_freq)
+    self.old_tx_freq = tx_freq
+    try:		# Look up low pass filter number based on MHz
+      lpf = self.lpfnum[tx_freq // 1000000]
+    except IndexError:
+      lpf = 0
+    try:		# Look up high pass filter number based on MHz
+      hpf = self.hpfnum[tx_freq // 1000000]
+    except IndexError:
+      hpf = 0
+    if DEBUG: print ("V2 filter LPF %d HPF %d" % (lpf, hpf))
+    lpf = 1 << lpf
+    hpf = 1 << hpf
+    if self.preamp:
+      hpf |= 0b10000000
+    self.want_data =  bytearray((lpf, hpf))
+    self.timer = 0
+  def HeartBeat(self):
+    global gatewayTime
+    if not self.socket:
+      return
+    try:	# The V2 filter box echoes its commands
+      self.have_data = self.socket.recv(50)
+    except socket.error:
+      pass
+    except socket.timeout:
+      pass
+    if self.have_data != self.want_data and time.time() - gatewayTime > gatewayLimit:
+      gatewayTime = time.time()
+      if self.timer <= 10:
+        self.timer += 1
+        if self.timer == 10:
+          print ('V2 filter box error')
+      try:
+        self.socket.send(self.want_data)
+        #print ('V2 filter box send data 0x%X 0x%X' % (ord(self.want_data[0]), ord(self.want_data[1])))
+      except socket.error:
+        pass
+      except socket.timeout:
+        pass
+
 class AntennaControl:	# Control my KI8BV dipole
   AntCtrlAddress = ('192.168.1.194', 0x3A00 + 33)
   def __init__(self, app, conf):
@@ -202,18 +358,18 @@ class AntennaControl:	# Control my KI8BV dipole
     self.socket.setblocking(0)
     self.socket.connect(self.AntCtrlAddress)
     self.have_data = None
-    self.want_data = '\00'
+    self.want_data = b'\00'
     self.timer = 0
   def SetTxFreq(self, tx_freq):
     self.timer = 0
     if tx_freq < 19000000:
-      self.want_data = '\03'
+      self.want_data = b'\03'
     elif tx_freq < 22000000:
-      self.want_data = '\02'
+      self.want_data = b'\02'
     elif tx_freq < 26500000:
-      self.want_data = '\01'
+      self.want_data = b'\01'
     else:
-      self.want_data = '\00'
+      self.want_data = b'\00'
   def HeartBeat(self):
     global gatewayTime
     try:	# The antenna control box echoes its commands
@@ -230,7 +386,7 @@ class AntennaControl:	# Control my KI8BV dipole
         self.timer = 0
       try:
         self.socket.send(self.want_data)
-        # print ("Change dipole to ord %d" % ord(self.want_data))
+        #print ("Change dipole to ord %d" % ord(self.want_data))
       except socket.error:
         pass
       except socket.timeout:
@@ -242,32 +398,32 @@ class AntennaTuner:     # Control my homebrew antenna tuner and my KI8BV dipole
     self.application = app	# Application instance (to provide attributes)
     self.conf = conf		# Config file module
     self.socket = None
-    self.GUI = None         # Control window
-    self.have_data = chr(255) * 3
+    self.have_data = b'\xFF\xFF\xFF'
     self.tx_freq = 0
     self.tune_freq = 0		# Frequency we last tuned
-    self.disable_freq_control = False
     self.timer = 0
     self.set_L = 0
     self.set_C = 0
     self.set_HighZ = 0
     self.antnum = 0				# Antenna number 0 or 1
-    self.fix_antnum = False		# Fix the antenna number
     self.dipole2 = AntennaControl(app, conf)	# Control the KI8BV dipole
+    path = 'TunerLCZ.txt'
     if sys.platform == "win32":
-      path = 'C:/pub/TunerLCZ.pkl'
+      self.path = 'C:/pub/' + path
     else:
-      path = '/home/jim/pub/TunerLCZ.pkl'
-    try:
-      fp = open(path, "r")
-      self.TunerLC = cPickle.load(fp)
-      fp.close()
-    except IOError:
-      self.TunerLC = {}
-    if hasattr(self.application, 'is_vna_program'):
-      self.is_vna = True
-    else:
-      self.is_vna = False
+      self.path = '/home/jim/pub/' + path
+    fp = open(self.path, "r")
+    lines = fp.readlines()
+    fp.close()
+    self.TunerLCZ = []
+    for line in lines:
+      freq, antL, antC, hilo = line.split()
+      freq = int(freq)
+      antL = int(antL)
+      antC = int(antC)
+      hilo = int(hilo)
+      self.TunerLCZ.append((freq, antL, antC, hilo))
+    self.TunerLCZ.sort()
     self.WantData()
   def WantData(self):
     if self.set_HighZ:
@@ -276,70 +432,68 @@ class AntennaTuner:     # Control my homebrew antenna tuner and my KI8BV dipole
       flags = 0x08
     if self.antnum:
       flags |= 0x04
-    self.want_data = chr(self.set_C) + chr(self.set_L) + chr(flags)
-    if self.set_HighZ:
-      hilo = 'HiZ'
-    else:
-      hilo = 'LoZ'
-    text = "L %3d,    C %3d,    %s,    Antenna %d" % (self.set_L, self.set_C, hilo, self.antnum)
-    if self.GUI:
-      self.GUI.sliderL.SetValue(self.set_L)
-      self.GUI.sliderC.SetValue(self.set_C)
-      self.GUI.UpdateText(text)
-    elif self.application.bottom_widgets:
-      self.application.bottom_widgets.UpdateText(text)
+    self.want_data = bytearray((self.set_C, self.set_L, flags))
   def open(self):
     self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
     self.socket.setblocking(0)
     self.socket.connect(self.address)
-    if self.is_vna:
-      self.GUI = AntTunerGUI(self, self.application)
-      self.GUI.Show()
   def close(self):
     pass
   def OnSpot(self, level):
+    # level is -1 for Spot button Off; else the Spot level 0 to 1000.
     pass
   def ChangeBand(self, band):
     pass
-  def SetTxFreq(self, tx_freq):
+  def SaveLCZ(self):
+    fp = open(self.path, 'w')
+    for tup in self.TunerLCZ:
+      fp.write("%12d %2d %2d %d\n" % tup)
+    fp.close()
+  def FindIndexTup(self, tx_freq):
+    i1 = 0
+    i2 = len(self.TunerLCZ) - 1
+    while 1:	# binary partition
+      i = (i1 + i2) // 2
+      if self.TunerLCZ[i][0] < tx_freq:
+        i1 = i
+      else:
+        i2 = i
+      if i2 - i1 <= 1:
+        break
+    # The correct setting is between i1 and i2, or below i1 or above i2.
+    # Choose the index that is closest in frequency.
+    if abs(tx_freq - self.TunerLCZ[i1][0]) < abs(tx_freq - self.TunerLCZ[i2][0]):
+      index = i1
+    else:
+      index = i2
+    if DEBUG: print ("StatuinControlGUI FindIndexTup", tx_freq, i1, i2, len(self.TunerLCZ), index, self.TunerLCZ[index])
+    return index, self.TunerLCZ[index]
+  def SetTxFreq(self, tx_freq, no_tune=False):
+    #if DEBUG: print("AntennaTuner SetTxFreq", tx_freq)
     self.tx_freq = tx_freq
     if tx_freq is None:
       return
-    if not self.fix_antnum:
-      if tx_freq < 17000000:
-        self.antnum = 0
-      else:
-        self.antnum = 1
-    if self.antnum == 1:
+    if tx_freq < 17000000:
+      if DEBUG and self.antnum == 1: print ("antnum 0")
+      self.antnum = 0
+    elif self.dipole2:
+      if DEBUG and self.antnum == 0: print ("antnum 1")
+      self.antnum = 1
+    if self.antnum == 1 and self.dipole2:
       self.dipole2.SetTxFreq(tx_freq)
     if not self.socket:
       return
-    if self.disable_freq_control:
+    if no_tune:		# Change dipole but don't tune the antenna tuner
       return
-    freq = (tx_freq + 5000) // 10000 * 10000		# rounded frequency
-    try:
-      newL, newC, newH = self.TunerLC[freq]
-    except KeyError:
-      newL, newC, newH = (0, 0, 0)
+    if abs(self.tune_freq - tx_freq) < 5000:		# ignore small changes
+      return
+    self.tune_freq = tx_freq
+    index, (f, newL, newC, newH) = self.FindIndexTup(tx_freq)
     if newH != self.set_HighZ or self.set_C != newC or self.set_L != newL:
-      # if not VNA, ignore small changes in frequency
-      if self.is_vna or abs(self.tune_freq - tx_freq) > 5000:
-        self.tune_freq = tx_freq
-        self.set_HighZ = newH
-        self.set_C = newC
-        self.set_L = newL
-        self.WantData()
-  def OnAntTuner(self, text):	# One of the tuner buttons was pressed
-    if text == 'Antenna':
-      self.fix_antnum = False
-    elif text == 'Ant 0':
-      self.fix_antnum = True
-      self.antnum = 0
-    elif text == 'Ant 1':
-      self.fix_antnum = True
-      self.antnum = 1
-    self.SetTxFreq(self.tx_freq)
-    self.WantData()
+      self.set_HighZ = newH
+      self.set_C = newC
+      self.set_L = newL
+      self.WantData()
   def HeartBeat(self):
     global gatewayTime
     if not self.socket:
@@ -359,9 +513,8 @@ class AntennaTuner:     # Control my homebrew antenna tuner and my KI8BV dipole
         self.timer = 0
     else:
       self.timer = 0
-    self.dipole2.HeartBeat()
-    if self.GUI:
-      self.GUI.HeartBeat()
+    if self.dipole2:
+      self.dipole2.HeartBeat()
   def Send(self):
     try:
       self.socket.send(self.want_data)
@@ -370,167 +523,251 @@ class AntennaTuner:     # Control my homebrew antenna tuner and my KI8BV dipole
     except socket.timeout:
       pass
 
-class AntTunerGUI(wx.Frame):    # Display a control window for my antenna tuner
-  def __init__(self, anttuner, app):
-    wx.Frame.__init__(self, app.main_frame, title="Antenna Tuner Control",
-       style=wx.STAY_ON_TOP|wx.CAPTION)
-    self.anttuner = anttuner
+class StationControlGUI(wx.Frame):    # Display a stand-alone control window for my antenna tuner
+  def __init__(self, frame, hware, app, conf):
+    wx.Frame.__init__(self, parent=frame, title="Station Control",
+       style=wx.CAPTION|wx.CLOSE_BOX)
+    self.hware = hware
     self.application = app
+    self.conf = conf
     self.data_saving = False
-    self.SetBackgroundColour('white')
+    self.tx_freq = 0
+    self.anttuner = self.hware.anttuner
+    self.SetBackgroundColour('light steel blue')
+    self.Bind(wx.EVT_CHAR_HOOK, self.OnKeyDown)
     sizer = wx.GridBagSizer(hgap=5, vgap=3)
-    font = wx.Font(12, wx.FONTFAMILY_SWISS, wx.NORMAL, wx.FONTWEIGHT_NORMAL, face=conf.quisk_typeface)
-    is_vna = anttuner.is_vna
-    b = wx.StaticText(self, -1, "Adjust L", style=wx.ALIGN_CENTER)
+    font = wx.Font(12, wx.FONTFAMILY_SWISS, wx.NORMAL, wx.FONTWEIGHT_NORMAL, False, conf.quisk_typeface)
+    row = 0
+    szr = wx.BoxSizer(wx.HORIZONTAL)
+    self.bands = []
+    for band in ('160', '80', '60', '40', '30', '20', '17', '15', '12', '10'):
+      b = wx.lib.buttons.GenToggleButton(self, -1, band)
+      b.Enable(False)
+      b.SetUseFocusIndicator(False)
+      b.SetBezelWidth(4)
+      self.bands.append(b)
+      szr.Add(b)
+      self.Bind(wx.EVT_BUTTON, self.OnBtnBand, b)
+    sizer.Add(szr, pos=(row, 0), span=(1, 3), flag=wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.RIGHT|wx.TOP, border=20)
+    row = 1
+    b = wx.StaticText(self, -1, "    Tuner Adjust L   ", style=wx.ALIGN_CENTER)
     b.SetFont(font)
-    sizer.Add(b, pos=(0, 0))
+    sizer.Add(b, pos=(row, 0), flag=wx.ALIGN_CENTER_VERTICAL)
     b = self.sliderL = wx.Slider(self, -1, 0, 0, 255, size=(700, -1), style=wx.SL_HORIZONTAL|wx.SL_LABELS)
-    b.Enable(is_vna)
     b.Bind(wx.EVT_SCROLL, self.OnSliderL)
-    sizer.Add(b, pos=(0,1))
-    b = wx.Button(self, -1, "HiLo")
+    sizer.Add(b, pos=(row,1))
+    b = self.btnHiLo = wx.lib.buttons.GenToggleButton(self, -1, "HiLo")
+    b.SetUseFocusIndicator(False)
+    b.SetBezelWidth(4)
     b.SetFont(font)
-    b.Enable(is_vna)
     self.Bind(wx.EVT_BUTTON, self.OnButtonHiLo, b)
-    sizer.Add(b, pos=(0,2))
-    b = wx.StaticText(self, -1, "Adjust C")
+    sizer.Add(b, pos=(row,2), flag=wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.RIGHT, border=20)
+    row = 2
+    b = wx.StaticText(self, -1, "    Tuner Adjust C   ")
     b.SetFont(font)
-    sizer.Add(b, pos=(1, 0))
+    sizer.Add(b, pos=(row, 0), flag=wx.ALIGN_CENTER_VERTICAL)
     b = self.sliderC = wx.Slider(self, -1, 0, 0, 255, size=(700, -1), style=wx.SL_HORIZONTAL|wx.SL_LABELS)
-    b.Enable(is_vna)
     b.Bind(wx.EVT_SCROLL, self.OnSliderC)
-    sizer.Add(b, pos=(1,1))
-    b = wx.Button(self, -1, "Save")
+    sizer.Add(b, pos=(row,1), flag=wx.ALIGN_CENTER_VERTICAL)
+    b = wx.lib.buttons.GenButton(self, -1, "Save")
+    b.SetUseFocusIndicator(False)
+    b.SetBezelWidth(4)
     b.SetFont(font)
-    b.Enable(is_vna)
     self.Bind(wx.EVT_BUTTON, self.OnButtonSave, b)
-    sizer.Add(b, pos=(1,2))
-    b = wx.StaticText(self, -1, "Status", style=wx.ALIGN_CENTER)
+    sizer.Add(b, pos=(row,2), flag=wx.ALIGN_CENTER_VERTICAL|wx.LEFT|wx.RIGHT, border=20)
+    row = 3
+    szr = wx.BoxSizer(wx.HORIZONTAL)
+    szr.AddSpacer(20)
+    b = wx.lib.buttons.GenToggleButton(self, -1, "Key Down")
+    b.Enable(False)
     b.SetFont(font)
-    sizer.Add(b, pos=(3, 0))
-    self.status = b = wx.StaticText(self, -1, "Ready", style=wx.ST_NO_AUTORESIZE)
+    b.SetUseFocusIndicator(False)
+    b.SetBezelWidth(4)
+    self.Bind(wx.EVT_BUTTON, self.OnBtnKey, b)
+    szr.Add(b)
+    szr.AddSpacer(10)
+    b = wx.lib.buttons.GenToggleButton(self, -1, "FilterV2 Preamp")
+    b.Enable(False)
     b.SetFont(font)
-    sizer.Add(b, pos=(3, 1), flag=wx.EXPAND)
-    b = wx.ToggleButton(self, -1, "Disable")
+    b.SetUseFocusIndicator(False)
+    b.SetBezelWidth(4)
+    if conf.use_rx_udp == 10:		# Hermes UDP protocol for Hermes-Lite2
+      b.Enable(False)
+    self.Bind(wx.EVT_BUTTON, self.OnBtnV2Preamp, b)
+    szr.Add(b)
+    b = wx.StaticText(self, -1, "     Freq MHz ")
     b.SetFont(font)
-    b.Enable(is_vna)
-    self.Bind(wx.EVT_TOGGLEBUTTON, self.OnBtnDisable, b)
-    sizer.Add(b, pos=(3, 2))
-    self.SetSizer(sizer)
-    self.Fit()
+    szr.Add(b, flag=wx.ALIGN_CENTER_VERTICAL)
+    szr.AddSpacer(10)
+    self.freq_entry = wx.StaticText(self, -1, "0000.000")
+    #self.freq_entry = wx.TextCtrl(self, value='0000.000', style=wx.TE_PROCESS_ENTER)
+    self.freq_entry.SetFont(font)
+    #self.Bind(wx.EVT_TEXT_ENTER, self.OnFreqEntry, self.freq_entry)
+    szr.Add(self.freq_entry, flag=wx.ALIGN_CENTER_VERTICAL)
+    szr.AddSpacer(10)
+    b = wx.lib.buttons.GenButton(self, -1, "Freq-")
+    b.SetUseFocusIndicator(False)
+    b.SetBezelWidth(4)
+    b.SetFont(font)
+    self.Bind(wx.EVT_BUTTON, self.OnButtonDown, b)
+    szr.Add(b)
+    szr.AddSpacer(10)
+    b = wx.lib.buttons.GenButton(self, -1, "Freq+")
+    b.SetUseFocusIndicator(False)
+    b.SetBezelWidth(4)
+    b.SetFont(font)
+    self.Bind(wx.EVT_BUTTON, self.OnButtonUp, b)
+    szr.Add(b)
+    szr.AddSpacer(10)
+    b = wx.lib.buttons.GenButton(self, -1, "Delete")
+    b.SetUseFocusIndicator(False)
+    b.SetBezelWidth(4)
+    b.SetFont(font)
+    self.Bind(wx.EVT_BUTTON, self.OnButtonDelete, b)
+    szr.Add(b)
+    sizer.Add(szr, pos=(row, 0), span=(1, 3))
+    if frame is None:
+      self.SetSizerAndFit(sizer)
+    else:
+      self.SetSizer(sizer)
+      self.Fit()
+    w, h = self.GetSize().Get()
+    self.SetSize((w + 20, h + 50))
+    self.Bind(wx.EVT_CLOSE, self.OnBtnClose)
+    if frame is None:	# Start a HeartBeat if the frame does not provide one
+      self.timer = wx.Timer(self)
+      self.Bind(wx.EVT_TIMER, self.HeartBeat)
+      self.timer.Start(100)
+  def SetTxFreq(self, tx_freq):
+    tx_freq = ((tx_freq + 500) // 1000) * 1000
+    self.tx_freq = tx_freq
+    anttuner = self.anttuner
+    index, tup = anttuner.FindIndexTup(tx_freq)
+    self.sliderL.SetValue(tup[1])
+    self.sliderC.SetValue(tup[2])
+    self.btnHiLo.SetValue(tup[3])
+    fff, anttuner.set_L, anttuner.set_C, anttuner.set_HighZ = tup
+    if tx_freq < 17000000:
+      anttuner.antnum = 0
+    elif anttuner.dipole2:
+      anttuner.antnum = 1
+    if anttuner.antnum == 1 and anttuner.dipole2:
+      anttuner.dipole2.SetTxFreq(tx_freq)
+    anttuner.WantData()
+    self.freq_entry.SetLabel("%.3f" % (tx_freq * 1E-6))
+  def OnBtnClose(self, event):
+    self.hware.anttuner.close()
+    self.hware.controlbox.close()
+    self.Destroy()
+  def HeartBeat(self, event):
+    self.hware.anttuner.HeartBeat()
+    self.hware.controlbox.HeartBeat()
+    if self.hware.v2filter:
+      self.hware.v2filter.HeartBeat()
+  def Open(self):	# Initialize the hardware
+    self.bands[0].SetToggle(True)	# 160 meters
+    self.hware.anttuner.open()
+    if self.hware.v2filter:
+      self.hware.v2filter.SetTxFreq(1900000)	# Start 160 meters
+  def OnKeyDown(self, event):
+    key = event.GetRawKeyCode()
+    if key == 76:	# L
+      self.sliderL.SetValue(self.sliderL.GetValue() + 1)
+      self.OnSliderL()
+    elif key == 108:	# l
+      self.sliderL.SetValue(self.sliderL.GetValue() - 1)
+      self.OnSliderL()
+    elif key == 67:	# C
+      self.sliderC.SetValue(self.sliderC.GetValue() + 1)
+      self.OnSliderC()
+    elif key == 99:	# c
+      self.sliderC.SetValue(self.sliderC.GetValue() - 1)
+      self.OnSliderC()
+    else:
+      event.Skip()
+  def OnBtnBand(self, event):
+    btn = event.GetEventObject()
+    for band in self.bands:
+      if band == btn:
+        band.SetToggle(True)
+      else:
+        band.SetToggle(False)
+    band = btn.GetLabel()
+    f1, f2 = self.conf.BandEdge[band]
+    f = (f1 + f2) // 2
+    f = ((f + 500) // 1000) * 1000
+    if self.hware.v2filter:
+      self.hware.v2filter.SetTxFreq(f)
+    self.anttuner.SetTxFreq(f, no_tune=True)
+    self.freq_entry.ChangeValue("%.3f" % (f * 1E-6))
+    self.SetTxFreq(f)
   def OnButtonHiLo(self, event):
-    anttuner = self.anttuner
-    if anttuner.set_HighZ:
-      anttuner.set_HighZ = 0
+    if self.btnHiLo.GetValue():
+      self.anttuner.set_HighZ = 1
     else:
-      anttuner.set_HighZ = 1
-    anttuner.WantData()
+      self.anttuner.set_HighZ = 0
+    self.anttuner.WantData()
+  def OnFreqEntry(self, event=None):
+    freq = self.freq_entry.GetValue()
+    freq = float(freq) * 1E6
+    freq = int(freq + 0.1)
+    self.tx_freq = freq
+  def OnButtonSearch(self, event):
+    freq = self.freq_entry.GetValue()
+    freq = float(freq) * 1E6
+    freq = int(freq + 0.1)
+    self.tx_freq = freq
+    self.SetTxFreq(self.tx_freq)
+  def OnButtonDown(self, event):
+    index, tup = self.anttuner.FindIndexTup(self.tx_freq)
+    index -= 1
+    if index >= 0:
+      self.SetTxFreq(self.anttuner.TunerLCZ[index][0])
+  def OnButtonUp(self, event):
+    index, tup = self.anttuner.FindIndexTup(self.tx_freq)
+    index += 1
+    if index < len(self.anttuner.TunerLCZ):
+      self.SetTxFreq(self.anttuner.TunerLCZ[index][0])
+  def OnButtonDelete(self, event):
+    index, tup = self.anttuner.FindIndexTup(self.tx_freq)
+    if self.tx_freq == self.anttuner.TunerLCZ[index][0]:
+      del self.anttuner.TunerLCZ[index]
+    if index > 0:
+      self.SetTxFreq(self.anttuner.TunerLCZ[index - 1][0])
+    elif len(self.anttuner.TunerLCZ) > 0:
+      self.SetTxFreq(self.anttuner.TunerLCZ[0][0])
+    else:
+      self.SetTxFreq(0)
   def OnButtonSave(self, event):
-    self.data = []
-    self.data.append(tuple(self.application.graph.data_freq))
-    self.data_lcz = []
-    self.data_avg_freq = (self.application.graph.data_freq[0] + self.application.graph.data_freq[-1]) / 2
-    self.data_avg_freq = f = (self.data_avg_freq + 500000) / 1000000		# frequency rounded to megahertz
-    if f == 5:		# try to limit the number of L/C combinations
-      for HighZ in (0, ):
-        for L in range(0, 4):
-          for C in range(0, 61):
-            self.data_lcz.append((L, C, HighZ))
-    elif f == 7:
-      for HighZ in (0, ):
-        for L in range(0, 21):
-          for C in range(0, 91):
-            self.data_lcz.append((L, C, HighZ))
-    elif f == 10:
-      for HighZ in (0, ):
-        for L in range(0, 8):
-          for C in range(0, 91):
-            self.data_lcz.append((L, C, HighZ))
-    elif f == 14:
-      for HighZ in (0, ):
-        for L in range(0, 11):
-          for C in range(0, 21):
-            self.data_lcz.append((L, C, HighZ))
-    elif f == 18:
-      for HighZ in (0, ):
-        for L in range(0, 11):
-          for C in range(0, 11):
-            self.data_lcz.append((L, C, HighZ))
-    elif f == 21:
-      for HighZ in (0, ):
-        for L in range(0, 11):
-          for C in range(0, 11):
-            self.data_lcz.append((L, C, HighZ))
-    elif f == 25:
-      for HighZ in (0, 1):
-        for L in range(0, 11):
-          for C in range(0, 11):
-            self.data_lcz.append((L, C, HighZ))
-    elif 28 <= f <= 30:
-      for HighZ in (0, 1):
-        for L in range(0, 11):
-          for C in range(0, 15):
-            self.data_lcz.append((L, C, HighZ))
-    anttuner = self.anttuner
-    anttuner.set_L, anttuner.set_C, anttuner.set_HighZ = self.data_lcz[0]
-    del self.data_lcz[0]
-    anttuner.WantData()
-    self.data_saving = True
-    self.data_count = 0
-  def OnButtonSaveJIM(self, event):
-    anttuner = self.anttuner
-    freq = anttuner.tx_freq
-    freq = (freq + 5000) / 10000 * 10000      # round
-    LC = anttuner.TunerLC[anttuner.antnum]
-    for i in range(len(LC)):	# Record new freq and L/C
-      if abs(LC[i][0] - freq) < 1000:
-        LC[i] = (freq, anttuner.set_L, anttuner.set_C, anttuner.set_HighZ)
-        break
+    L = self.sliderL.GetValue()
+    C = self.sliderC.GetValue()
+    if self.btnHiLo.GetValue():
+      hilo = 1
     else:
-      LC.append((freq, anttuner.set_L, anttuner.set_C, anttuner.set_HighZ))
-      LC.sort()
-    self.WriteFile()
-  def UpdateText(self, text):
-    self.status.SetLabel(text)
-  def OnSliderL(self, event):
-    self.anttuner.set_L = event.GetEventObject().GetValue()
+      hilo = 0
+    tup = (self.tx_freq, L, C, hilo)
+    index, t = self.anttuner.FindIndexTup(self.tx_freq)
+    if self.tx_freq == self.anttuner.TunerLCZ[index][0]:
+      self.anttuner.TunerLCZ[index] = tup
+    else:
+      self.anttuner.TunerLCZ.append(tup)
+    self.anttuner.TunerLCZ.sort()
+    self.anttuner.SaveLCZ()
+  def OnSliderL(self, event=None):
+    self.anttuner.set_L = self.sliderL.GetValue()
     self.anttuner.WantData()
-  def OnSliderC(self, event):
-    self.anttuner.set_C = event.GetEventObject().GetValue()
+  def OnSliderC(self, event=None):
+    self.anttuner.set_C = self.sliderC.GetValue()
     self.anttuner.WantData()
-  def OnBtnDisable(self, event):
-    self.anttuner.disable_freq_control = event.GetEventObject().GetValue()
-  def WriteFile(self):
-    anttuner = self.anttuner
-    fp = open(anttuner.TunerLC_fname, 'w')
-    for i in (0, 1):          # antenna number
-      lst = anttuner.TunerLC[i]   # freq, L, C, IsHi for each antenna
-      for f, l, c, ishi in  lst:
-        fp.write("%d %9d %4d %4d  %1d\n" % (i, f, l, c, ishi))
-    fp.close()
-  def HeartBeat(self):
-    gr = self.application.graph
-    if self.data_saving:
-      anttuner = self.anttuner
-      if anttuner.have_data == anttuner.want_data:
-        self.data_count += 1
-        if self.data_count > 4:
-          self.data_count = 0
-          self.data.append(((anttuner.set_L, anttuner.set_C, anttuner.set_HighZ), tuple(self.application.graph.data_mag)))
-          #print ('record 500', (anttuner.set_L, anttuner.set_C, anttuner.set_HighZ), self.application.graph.data_mag[500])
-          if self.data_lcz:
-            anttuner.set_L, anttuner.set_C, anttuner.set_HighZ = self.data_lcz[0]
-            #print (self.data_lcz[0])
-            del self.data_lcz[0]
-            anttuner.WantData()
-          else:
-            self.data_saving = False
-            fp = open("/home/jim/pub/Tuner/data%2d.pkl" % self.data_avg_freq, "w")
-            cPickle.dump(self.data, fp)
-            fp.close()
+  def OnBtnV2Preamp(self, event):
+    if self.hware.v2filter:
+      self.hware.v2filter.SetPreamp(event.GetEventObject().GetValue())
+  def OnBtnKey(self, event):
+    self.hware.controlbox.SetKeyDown(event.GetEventObject().GetValue())
 
 class AT200PC:		# Control an AT-200PC autotuner made by LDG
   def __init__(self, app, conf):
+    import serial
     self.application = app	# Application instance (to provide attributes)
     self.conf = conf		# Config file module
     self.serial = None
@@ -741,7 +978,7 @@ class AT200PC:		# Control an AT-200PC autotuner made by LDG
     i1 = 0
     i2 = len(self.TunerLC) - 1
     while 1:	# binary partition
-      i = (i1 + i2) / 2
+      i = (i1 + i2) // 2
       if self.TunerLC[i][0] < tx_freq:
         i1 = i
       else:
@@ -777,9 +1014,9 @@ class AT200PC:		# Control an AT-200PC autotuner made by LDG
   def ChangeBand(self, band):
     pass ##self.ReqSetFreq(self.tx_freq)
   def OnSpot(self, level):
-    # level 0 == OFF, else the level 10 to 1000
+    # level is -1 for Spot button Off; else the Spot level 0 to 1000.
     if self.serial:
-      if level == 0:
+      if level < 0:
         self.live_update = 0
       elif not self.live_update:
         self.live_update = 1
