@@ -290,9 +290,8 @@ extern int quiskImdLevel;				// level for rxMode IMD
 extern int quiskTxHoldState;			// state machine for Tx wait for repeater frequency shift
 extern double quisk_ctcss_freq;			// frequency in Hertz
 extern unsigned char quisk_pc_to_hermes[17 * 4];		// Data to send from the PC to the Hermes hardware
-extern unsigned char quisk_hermeslite_writequeue[4 * 5];	// One-time writes to Hermes-Lite
-extern unsigned int quisk_hermeslite_writepointer;		// write pointer into write queue, nonzero value triggers writes, 
-extern unsigned int quisk_hermeslite_writeattempts;		// counter for write retries
+extern unsigned char quisk_hermeslite_writequeue[5];		// One-time writes to Hermes-Lite
+extern unsigned int quisk_hermeslite_writepointer;		// 0==No request; 1=Send writequeue; 2==Wait for ACK; 3==0x3F error from HL2
 extern unsigned int quisk_hermes_code_version;			// Hermes code version from Hermes to PC
 extern unsigned int quisk_hermes_board_id;			// Hermes board ID from Hermes to PC
 extern int hermes_mox_bit;					// Hermes mox bit from the PC to Hermes
@@ -313,13 +312,11 @@ extern int quisk_active_sidetone;		// Whether and how to generate a sidetone
 extern int quisk_isFDX;				// Are we in full duplex mode?
 extern int quisk_use_serial_port;		// Are we using the serial port for CW key or PTT?
 extern play_state_t quisk_play_state;		// startup, receiving, sidetone
-extern int freedv_current_mode;			// current FreeDV mode; 700D, @)@) etc.
-extern int n_modem_sample_rate;			// Receive data, decimate to modem_sample_rate, FreeDV codec output data at speech_sample_rate
-extern int n_speech_sample_rate;		// Microphone decimate to speech_sample_rate, Freedv codec output data at modem_sample_rate, interpolate to 48000
 extern int quisk_start_cw_delay;		// milliseconds to delay output on serial or MIDI CW key down
 extern int quisk_start_ssb_delay;		// milliseconds to discard output for all modes except CW
 extern struct sound_dev * quiskPlaybackDevices[];	// array of Playback sound devices
 extern int quisk_close_file_play;
+extern double digital_output_level;
 
 extern PyObject * quisk_set_spot_level(PyObject * , PyObject *);
 extern PyObject * quisk_get_tx_filter(PyObject * , PyObject *);
@@ -338,20 +335,14 @@ extern PyObject * quisk_sound_errors(PyObject *, PyObject *);
 extern PyObject * quisk_set_file_record(PyObject *, PyObject *);
 extern PyObject * quisk_set_file_name(PyObject *, PyObject *, PyObject *);
 extern PyObject * quisk_set_tx_audio(PyObject *, PyObject *, PyObject *);
+extern PyObject * quisk_get_tx_audio(PyObject *, PyObject *);
 extern PyObject * quisk_is_vox(PyObject *, PyObject *);
 extern PyObject * quisk_set_udp_tx_correct(PyObject *, PyObject *);
 extern PyObject * quisk_set_hermes_filter(PyObject *, PyObject *);
 extern PyObject * quisk_set_alex_hpf(PyObject *, PyObject *);
 extern PyObject * quisk_set_alex_lpf(PyObject *, PyObject *);
 
-extern PyObject * quisk_freedv_open(PyObject *, PyObject *);
-extern PyObject * quisk_freedv_close(PyObject *, PyObject *);
-extern PyObject * quisk_freedv_get_snr(PyObject *, PyObject *);
-extern PyObject * quisk_freedv_get_version(PyObject *, PyObject *);
-extern PyObject * quisk_freedv_get_rx_char(PyObject *, PyObject *);
-extern PyObject * quisk_freedv_set_options(PyObject *, PyObject *, PyObject *);
 extern PyObject * quisk_set_sparams(PyObject *, PyObject *, PyObject *);
-extern PyObject * quisk_freedv_set_squelch_en(PyObject *, PyObject *);
 extern PyObject * quisk_open_key(PyObject *, PyObject *, PyObject *);
 extern PyObject * quisk_close_key(PyObject *, PyObject *);
 extern PyObject * quisk_set_sound_name(PyObject *, PyObject *);
@@ -361,7 +352,7 @@ extern PyObject * quisk_wasapi_control_midi(PyObject *, PyObject *, PyObject *);
 // WDSP interface
 #define QUISK_WDSP_RX	1
 extern PyObject * quisk_wdsp_set_parameter(PyObject *, PyObject *, PyObject *);
-extern int wdspFexchange0(int, double *, int);
+extern int wdspFexchange0(int, complex double *, int);
 
 // These function pointers are the Start/Stop/Read interface for
 // the SDR-IQ and any other C-language extension modules that return
@@ -395,7 +386,6 @@ void quisk_file_playback(complex double *, int, double);
 void quisk_tmp_playback(complex double *, int, double);
 void quisk_hermes_tx_send(int, int *);
 void quisk_udp_mic_error(char *);
-void quisk_check_freedv_mode(void);
 void quisk_calc_audio_graph(double, complex double *, double *, int, int);
 double QuiskDeltaSec(int);
 void * quisk_make_sidetone(struct sound_dev *, int);
@@ -403,12 +393,7 @@ void * quisk_make_txIQ(struct sound_dev *, int);
 int quisk_play_sidetone(struct sound_dev *);
 void quisk_set_play_state(void);
 void quisk_poll_hardware_key(void);
-
-// Functions supporting digital voice codecs
-typedef int  (* ty_dvoice_codec_rx)(complex double *, double *, int, int);
-typedef int  (* ty_dvoice_codec_tx)(complex double *, double *, int);
-extern ty_dvoice_codec_rx  pt_quisk_freedv_rx;
-extern ty_dvoice_codec_tx  pt_quisk_freedv_tx;
+void PreDistort(complex double * amp_in_samples, complex double * amp_out_samples, int nSamples, complex double * tx_samples, int num_tx);
 
 // Driver function definitions=================================================
 int  quisk_read_alsa(struct sound_dev *, complex double *);
@@ -465,7 +450,7 @@ int import_quisk_api(void);	// used to initialize Quisk_API
 #define QuiskSleepMicrosec	(*(	void	(*)	(int)			)Quisk_API[5])
 #define QuiskPrintTime		(*(	void	(*)	(const char *, int)	)Quisk_API[6])
 #define quisk_sample_source	(*(	void	(*)	(ty_sample_start, ty_sample_stop, ty_sample_read)	)Quisk_API[7])
-#define quisk_dvoice_freedv	(*(	void	(*)	(ty_dvoice_codec_rx, ty_dvoice_codec_tx)        	)Quisk_API[8])
+#define quisk_dvoice_freedv	(*(	void	(*)	(void)	        	)Quisk_API[8])
 #define quisk_is_key_down	(*(	int	(*)	(void)			                                )Quisk_API[9])
 #define quisk_sample_source4	(*(	void	(*)	(ty_sample_start, ty_sample_stop, ty_sample_read, ty_sample_write)	)Quisk_API[10])
 #define strMcpy                 (*(     char *  (*)     (char *, const char *, size_t)                          )Quisk_API[11])
@@ -480,7 +465,7 @@ double	QuiskTimeSec(void);
 void	QuiskSleepMicrosec(int);
 void	QuiskPrintTime(const char *, int);
 void	quisk_sample_source(ty_sample_start, ty_sample_stop, ty_sample_read);
-void	quisk_dvoice_freedv(ty_dvoice_codec_rx, ty_dvoice_codec_tx);
+void	quisk_dvoice_freedv(void);
 int	quisk_is_key_down(void);
 void	quisk_sample_source4(ty_sample_start, ty_sample_stop, ty_sample_read, ty_sample_write);
 char *  strMcpy(char *, const char *, size_t);
