@@ -289,6 +289,8 @@ class SliderBoxH:
       if application.remote_control_head:
         application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetValue()}\n')
       self.handler(None)
+  def SetFocus(self):
+    self.slider.SetFocus()
 
 class SliderBoxHH(SliderBoxH, wx.BoxSizer):
   """A horizontal control with a slider and text with a value.  The text must have a %d if display is True."""
@@ -580,6 +582,7 @@ class QuiskPushbutton(QuiskButtons, wx.lib.buttons.GenButton):
     self.Bind(wx.EVT_BUTTON, self.OnButton)
     self.InitButtons(text, text_color)
     self.direction = 1
+    self.normal_brush = self.up_brush
     if use_right:
       self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
       self.Bind(wx.EVT_RIGHT_UP, self.OnRightUp)
@@ -601,7 +604,12 @@ class QuiskPushbutton(QuiskButtons, wx.lib.buttons.GenButton):
       event = wx.PyEvent()
       event.SetEventObject(self)
       self.command(event)
-      
+  def SetColorNormal(self):
+    self.up_brush = self.normal_brush
+    self.Refresh()
+  def SetColor(self, color):
+    self.up_brush = wx.Brush(color)
+    self.Refresh()
 
 class QuiskRepeatbutton(QuiskButtons, wx.lib.buttons.GenButton):
   """A push button that repeats when held down."""
@@ -797,16 +805,19 @@ class QFilterButtonWindow(wx.Frame):
     x, y = wrap.GetPosition().Get()
     x, y = wrap.GetParent().ClientToScreen(wx.Point(x, y))
     w, h = wrap.GetSize()
-    height = h * 10
+    height = h * 12
     size = (w, height)
     if sys.platform == 'win32':
-      pos = (x, y - height)
+      pos = (x, y - height - h)
       t = 'Filter'
     else:
       pos = (x, y - height - h)
       t = ''
     wx.Frame.__init__(self, wrap.GetParent(), -1, t, pos, size,
       wx.FRAME_TOOL_WINDOW|wx.FRAME_FLOAT_ON_PARENT|wx.CLOSE_BOX|wx.CAPTION|wx.SYSTEM_MENU)
+    self.SetSizeHints(w, height, w, height)
+    hbox = wx.BoxSizer(wx.VERTICAL)
+    self.SetSizer(hbox)
     self.SetBackgroundColour(conf.color_freq)
     self.Bind(wx.EVT_CLOSE, self.OnClose)
     try:
@@ -814,19 +825,18 @@ class QFilterButtonWindow(wx.Frame):
     except ValueError:
       index = 0
       self.wrap.button.slider_value = self.valuelist[0]
-    self.slider = wx.Slider(self, -1, index, 0, 100, (0, 0), (w//2, height), wx.SL_VERTICAL|wx.SL_INVERSE)
+    self.slider = wx.Slider(self, -1, index, 0, 100, style=wx.SL_VERTICAL|wx.SL_INVERSE)
+    hbox.Add(self.slider, flag=wx.CENTER, proportion=1)
     self.slider.Bind(wx.EVT_SCROLL, self.OnSlider)
     self.SetTitle("%d" % self.valuelist[index])
     self.Show()
     self.slider.SetFocus()
+    self.Fit()
   def OnSlider(self, event):
     index = self.slider.GetValue()
     value = self.valuelist[index]
     self.SetTitle("%d" % value)
     self.wrap.ChangeSlider(value)
-    #self.wrap.SetLabel(str(value))
-    #self.wrap.SetValue(True, True)
-    #application.filterAdjBw1 = value
   def OnClose(self, event):
     self.wrap.adjust = None
     self.Destroy()
@@ -838,25 +848,29 @@ class QSliderButtonWindow(wx.Frame):
     x, y = button.GetPosition().Get()
     x, y = button.GetParent().ClientToScreen(wx.Point(x, y))
     w, h = button.GetSize()
-    height = h * 10
+    height = h * 12
     size = (w, height)
     if sys.platform == 'win32':
-      pos = (x, y - height)
+      pos = (x, y - height - h)
     else:
       pos = (x, y - height - h)
     wx.Frame.__init__(self, button.GetParent(), -1, '', pos, size,
       wx.FRAME_TOOL_WINDOW|wx.FRAME_FLOAT_ON_PARENT|wx.CLOSE_BOX|wx.CAPTION|wx.SYSTEM_MENU)
+    self.SetSizeHints(w, height, w, height)
+    hbox = wx.BoxSizer(wx.VERTICAL)
+    self.SetSizer(hbox)
     self.SetBackgroundColour(conf.color_freq)
     self.Bind(wx.EVT_CLOSE, self.OnClose)
     self.slider = wx.Slider(self, -1, value,
-             self.button.slider_min, self.button.slider_max,
-             (0, 0), (w//2, height), wx.SL_VERTICAL|wx.SL_INVERSE)
+             self.button.slider_min, self.button.slider_max, style=wx.SL_VERTICAL|wx.SL_INVERSE)
+    hbox.Add(self.slider, flag=wx.CENTER, proportion=1)
     self.slider.Bind(wx.EVT_SCROLL, self.OnSlider)
     if self.button.display:
       value = float(value) / self.button.slider_max
       self.SetTitle("%6.3f" % value)
     self.Show()
     self.slider.SetFocus()
+    self.Fit()
   def OnSlider(self, event):
     value = self.slider.GetValue()
     if self.button.display:
@@ -1541,6 +1555,8 @@ class QuiskMenu(wx.Menu):
     item.Check(is_checked)
     self.InitData(item, handler)
     return item
+  def AppendSeparator(self):
+    wx.Menu.AppendSeparator(self)
   def IsItemChecked(self, item_text):
     nid = self.item_text2id[item_text]
     menu_item = self.FindItemById(nid)
